import React from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { message } from 'antd'
import { JsonFormModal } from 'components'
import { validateDescriptor as vd, config } from 'utils'

const { drbdMasterMode } = config.featureSwitch

const getModelLable = (key, index) => {
  const drbdModelLable = {
    main_node_name: ['主节点名', '节点1名'],
    main_node_ip: ['主节点IP', '节点1IP'],
    main_node_device: ['主节点设备', '节点1设备'],
    slave_node_name: ['从节点名', '节点2名'],
    slave_node_ip: ['从节点IP', '节点2IP'],
    slave_node_device: ['从节点设备', '节点2设备'],
  }

  return drbdModelLable[key][index]
}

class DrbdSetting extends React.PureComponent {
  constructor (props) {
    super(props)

    this.onOk = this.onOk.bind(this)
    this.onCancel = this.onCancel.bind(this)
  }
  onOk (data) {
    const submitData = {
      drbd_type: '0',
      ...data,
    }
    if (submitData.flag && (submitData.main_node_name === submitData.slave_node_name || submitData.main_node_ip === submitData.slave_node_ip)) {
      message.error('DRBD两个节点的节点名/IP不能相同', 10)
      return
    }
    let submitUrl = submitData.flag ? 'cluster/updateDrbd' : 'cluster/deleteDrbd'
    delete submitData.flag
    this.props.dispatch({ type: submitUrl, payload: { ...submitData } }).then((resp) => {
      const infoDom = <span dangerouslySetInnerHTML={{ __html: resp.info }} />
      message.success(infoDom)
      // 配置成功后同步更新资源列表
      this.props.dispatch({ type: 'resource/query' })
      this.onCancel()
    })
  }
  onCancel () {
    this.props.dispatch({ type: 'cluster/updateState', payload: { drbdModalVisible: false } })
  }
  getSelectChildrenData () {
    let { nodeList } = this.props
    return nodeList
      .map((node) => {
        return {
          label: node.id,
          value: node.id,
        }
      })
  }

  getFormDataModel () {
    let { formModel, dispatch } = this.props
    let labelIndex = 0
    let drbdSwitch = {
      flag: {
        label: '启用DRBD',
        type: 'Switch',
        initialValue: formModel.flag,
        props: {
          defaultChecked: formModel.flag,
          onChange: (value) => {
            dispatch({ type: 'cluster/updateState', payload: { drbdModel: { ...formModel, flag: value } } })
          },
        },
      },
    }
    if (drbdMasterMode) {
      labelIndex = parseInt(formModel.drbd_type, 10)
      drbdSwitch.drbd_type = {
        label: '模式',
        type: 'RadioGroup',
        props: {
          disabled: !formModel.flag,
          onChange: (e) => {
            dispatch({ type: 'cluster/updateState', payload: { drbdModel: { ...formModel, drbd_type: e.target.value } } })
          },
        },
        initialValue: formModel.drbd_type,
        children: [{
          label: '主备',
          value: '0',
        }, {
          label: '主主',
          value: '1',
        }],
      }
    }

    return {
      ...drbdSwitch,
      main_node_name: {
        label: getModelLable('main_node_name', labelIndex),
        type: 'Select',
        initialValue: formModel.main_node_name,
        rules: formModel.flag ? [vd.required] : [{}],
        props: {
          disabled: !formModel.flag,
          placeholder: '请选择',
        },
        children: this.getSelectChildrenData(),
      },
      main_node_ip: {
        label: getModelLable('main_node_ip', labelIndex),
        type: 'Input',
        initialValue: formModel.main_node_ip,
        rules: formModel.flag ? [vd.required, vd.ip] : [{}],
        props: {
          disabled: !formModel.flag,
        },
      },
      main_node_device: {
        label: getModelLable('main_node_device', labelIndex),
        type: 'Input',
        initialValue: formModel.main_node_device,
        rules: formModel.flag ? [vd.required, vd.devPath, vd.validPath] : [{}],
        props: {
          disabled: !formModel.flag,
        },
      },
      slave_node_name: {
        label: getModelLable('slave_node_name', labelIndex),
        type: 'Select',
        initialValue: formModel.slave_node_name,
        rules: formModel.flag ? [vd.required] : [{}],
        props: {
          disabled: !formModel.flag,
          placeholder: '请选择',
        },
        children: this.getSelectChildrenData(),
      },
      slave_node_ip: {
        label: getModelLable('slave_node_ip', labelIndex),
        type: 'Input',
        initialValue: formModel.slave_node_ip,
        rules: formModel.flag ? [vd.required, vd.ip] : [{}],
        props: {
          disabled: !formModel.flag,
        },
      },
      slave_node_device: {
        label: getModelLable('slave_node_device', labelIndex),
        type: 'Input',
        initialValue: formModel.slave_node_device,
        rules: formModel.flag ? [vd.required, vd.devPath, vd.validPath] : [{}],
        props: {
          disabled: !formModel.flag,
        },
      },
      dir: {
        label: '挂载目录',
        type: 'Input',
        initialValue: formModel.dir,
        rules: formModel.flag ? [vd.required, vd.absPath, vd.validPath] : [{}],
        props: {
          disabled: !formModel.flag,
        },
      },
    }
  }
  render () {
    const { updateConfirmLoading, deleteConfirmLoading, formModel, visible } = this.props
    if (!visible || !formModel) {
      return null
    }
    let modalOpts = {
      onOk: this.onOk,
      onCancel: this.onCancel,
      dataModel: this.getFormDataModel(formModel),
      visible,
      maskClosable: false,
      title: 'DRBD配置',
      style: { top: 47 },
      formItemProps: {
        labelCol: {
          span: 6,
        },
        wrapperCol: {
          span: 14,
        },

      },
      confirmLoading: formModel.flag ? updateConfirmLoading : deleteConfirmLoading,
    }

    return (
      <JsonFormModal {...modalOpts} />
    )
  }
}

DrbdSetting.propTypes = {
  dispatch: PropTypes.func,
  modalVisible: PropTypes.bool,
  formModel: PropTypes.object,
  updateConfirmLoading: PropTypes.bool,
  deleteConfirmLoading: PropTypes.bool,
}

const mapStateToProps = ({ cluster, loading, node }) => {
  if (!cluster.drbdModalVisible) {
    return {}
  }
  return {
    nodeList: node.list,
    visible: cluster.drbdModalVisible,
    formModel: cluster.drbdModel,
    updateConfirmLoading: loading.effects['cluster/updateDrbd'],
    deleteConfirmLoading: loading.effects['cluster/deleteDrbd'],
  }
}

export default connect(mapStateToProps)(DrbdSetting)
