/* global document, window */
import React from 'react'
import PropTypes from 'prop-types'
import classnames from 'classnames'
import { connect } from 'dva'
import { Layout, ClusterViews, LicenseView, SettingModal } from 'components'
import { openPages, logo, name, featureSwitch } from 'config'
import { Helmet } from 'react-helmet'
import { withRouter } from 'dva/router'
import { Spin, Modal } from 'antd'
import '../themes/index.less'
import './app.less'

const { Header, Sider, styles } = Layout
const { ClusterSetting, AlarmSetting, HeartBeatSetting, DrbdSetting, GfsSetting, ScriptGenerator, ClusterOprtInfo } = ClusterViews
const { diskHbMode } = featureSwitch

const App = ({ children, dispatch, app, loading, hbstatus, location, login }) => {
  const { siderFold, isNavbar, menuPopoverVisible } = app
  let { pathname } = location
  let license = login ? login.license : {}
  pathname = pathname.startsWith('/') ? pathname : `/${pathname}`

  const clickNavMenu = (item) => {
    switch (item.key) {
      case 'cluterSetting':
        dispatch({ type: 'cluster/query' }).then(() => {
          dispatch({ type: 'cluster/updateState', payload: { clusterSettingModalVisible: true } })
        })
        break
      case 'alarmSetting':
        dispatch({ type: 'cluster/queryAlarm' }).then(() => {
          dispatch({ type: 'cluster/updateState', payload: { alarmSettingModalVisible: true } })
        })
        break
      case 'heartBeatSetting':
        dispatch({ type: 'cluster/queryHeartBeat' }).then(() => {
          if (diskHbMode) {
            dispatch({ type: 'cluster/queryDevices' })
          }
          dispatch({ type: 'cluster/updateState', payload: { heartBeatModalVisible: true } })
        })
        break
      case 'DrbdSetting':
        dispatch({ type: 'cluster/queryDrbd' }).then(() => {
          dispatch({ type: 'cluster/updateState', payload: { drbdModalVisible: true } })
        })
        break
      case 'GfsSetting':
        dispatch({ type: 'cluster/queryGfs' }).then(() => {
          dispatch({ type: 'cluster/updateState', payload: { gfsModalVisible: true } })
        })
        break
      case 'logDownload':
        Modal.confirm({
          title: '日志下载需要一段时间，你确定要立即下载集群日志吗？',
          onOk: () => {
            return dispatch({ type: 'app/getLogs' }).then((resp) => {
              // 适用于IE10
              if (window.navigator.msSaveOrOpenBlob) {
                window.open(resp.data.filepath)
                return Promise.resolve()
              }

              let aLink = document.createElement('a')
              let evt = document.createEvent('MouseEvents')
              evt.initEvent('click', false, true)
              aLink.href = resp.data.filepath
              aLink.dispatchEvent(evt)

              return Promise.resolve()
            })
          },
          style: {
            top: 180,
          },
        })
        break
      case 'scriptGenerator':
        dispatch({ type: 'cluster/updateState', payload: { scriptModalVisible: true } })
        break
      case 'clusterOprtInfo':
        dispatch({ type: 'cluster/queryCommands' }).then(() => {
          dispatch({ type: 'cluster/updateState', payload: { commandModalVisible: true } })
        })
        break
      case 'resource':
        break
      default:
        throw new Error(`Unknown menu key: ${item.key}`)
    }
  }

  const clickTopMenu = (item) => {
    switch (item.key) {
      case 'setting':
        dispatch({ type: 'app/updateState', payload: { SettingModalVisible: true } })
        break
      case 'logout':
        dispatch({ type: 'app/logout' })
        break
      default:
        break
    }
  }

  const headerProps = {
    location,
    siderFold,
    isNavbar,
    menuPopoverVisible,
    clickNavMenu,
    license,
    clickTopMenu,
    isLicenseLoading: loading.effects['login/getLicense'],
    hbstatus,
    showLicense () {
      dispatch({ type: 'login/getLicense' }).then(() => {
        dispatch({ type: 'app/updateState', payload: { licenseModalVisible: true } })
      })
    },
    switchMenuPopover () {
      dispatch({ type: 'app/switchMenuPopver' })
    },
    switchSider () {
      dispatch({ type: 'app/switchSider' })
    },
  }

  const siderProps = {
    location,
    siderFold,
    clickNavMenu,
  }

  if (openPages && openPages.includes(pathname)) {
    return (<div>
      {children}
    </div>)
  }

  let isLoading = loading.effects['cluster/query'] ||
                  loading.effects['cluster/queryAlarm'] ||
                  loading.effects['cluster/queryHeartBeat'] ||
                  loading.effects['resource/detail'] ||
                  loading.effects['resource/queryRscType'] ||
                  loading.effects['resource/queryRscRestriction'] ||
                  loading.effects['cluster/queryDrbd'] ||
                  loading.effects['cluster/queryGfs'] ||
                  loading.effects['cluster/queryCommands']
  return (
    <div>
      <Helmet>
        <title>{name}</title>
        <meta name="viewport" content="width=device-width, initial-scale=1.0" />
        <link rel="icon" href={logo} type="image/x-icon" />
      </Helmet>
      <div className={classnames(styles.layout, { [styles.fold]: isNavbar ? false : siderFold }, { [styles.withnavbar]: isNavbar })}>
        {!isNavbar ? <aside className={styles.sider}>
          <Sider {...siderProps} />
        </aside> : ''}
        <div className={styles.main}>
          <Header {...headerProps} />
          <div className={styles.container}>
            <div className={styles.content}>
              <Spin spinning={Boolean(isLoading)} delay={100} size="large">
                { children }
              </Spin>
            </div>
          </div>
        </div>
      </div>
      <ClusterSetting />
      <AlarmSetting />
      <HeartBeatSetting />
      <LicenseView />
      <DrbdSetting />
      <GfsSetting />
      <SettingModal />
      <ScriptGenerator />
      <ClusterOprtInfo />
    </div>
  )
}


App.propTypes = {
  children: PropTypes.element.isRequired,
  location: PropTypes.object,
  dispatch: PropTypes.func,
  app: PropTypes.object,
  loading: PropTypes.object,
}

export default withRouter(connect(({ app, loading, cluster, login }) => ({ app, loading, hbstatus: cluster.hbstatus, login }))(App))
