/* global document, window */
import React from 'react'
import PropTypes from 'prop-types'
import classnames from 'classnames'
import { connect } from 'dva'
import { NodeView, RscDetailModal, RscRestrictions, RscMigrationModal, Loader } from 'components'
import { Avatar, Badge, Table, Button, Menu, Dropdown, Icon, Tooltip, Popconfirm, Modal } from 'antd'
import { strcmp, queryArray, handleOperation } from 'utils'
import styles from './index.less'

function parseNodeStatusMsg (node) {
  let running = true
  let standby = false

  switch (node.status) {
    case 'Master':
      break
    case 'Master/Standby':
      standby = true
      break
    case 'Not Running/Standby':
      running = false
      standby = true
      break
    case 'Standby':
      standby = true
      break
    case 'Not Running':
      running = false
      break
    case 'Running':
      running = true
      break
    default:
      break
  }
  node.running = running
  node.standby = standby
}

function parseRscStatusMsg (rsc) {
  if (rsc.drbdStatus && rsc.drbdStatus !== '0') {
    return 'error'
  }
  switch (rsc.status) {
    case 'Running':
      return 'success'
    case 'Not Running':
      if (rsc.status_message) {
        return 'error'
      }
      return 'default'

    case 'Unmanaged':
      return 'warning'
    case 'Failed':
    case 'Stop Failed':
      return 'error'
    case 'running(Master)':
    case 'running(Slave)':
      rsc.status = 'Running'
      return 'success'
    default:
      return 'default'
  }
}

class Resource extends React.PureComponent {
  componentDidUpdate (prevProps) {
    const { mainLockVisible } = this.props
    if (!prevProps.mainLockVisible && mainLockVisible) {
      this.showMainLockView()
    }
  }
  handleRowClick (record) {
    let dispatch = this.props.dispatch
    dispatch({
      type: 'resource/updateState',
      payload: {
        selectedRowKeys: [record.key],
        selectedRows: [record],
      },
    })
  }
  handleResourceOperation (id, action) {
    handleOperation(this.props.dispatch, 'resource', id, action)
  }
  handleReFresh () {
    let { dispatch } = this.props
    dispatch({ type: 'resource/query' })
    dispatch({ type: 'node/query' })
    dispatch({ type: 'cluster/queryHeartBeatStatus' })
  }
  // 当resource接口返回 请检查HA集群是否处于启动状态时，弹出mainlock，并触发定时刷新
  showMainLockView () {
    const { dispatch, autoReFreshId, autoReFreshTaskId } = this.props
    // 当系统定时任务开启时，先暂停该定时任务, 并设置定时任务的id为0
    if (autoReFreshId !== '0') {
      clearInterval(autoReFreshTaskId)
      dispatch({ type: 'app/updateState', payload: { autoReFreshTaskId: 0 } })
    }

    Modal.warning({
      title: '集群处于未启动状态。',
      content: (
        <div>
          <p>请启动集群后继续操作。</p>
        </div>
      ),
      okText: '启动',
      onOk: () => {
        return new Promise((resolve) => {
          dispatch({ type: 'node/operateLocal', payload: { action: 'start' } }).then(() => {
            setTimeout(() => {
              window.location = '/resource'
              return resolve
            }, 20000)
          })
        })
      },
    })
  }
  render () {
    let { rsclist, nodelist, selectedRowKeys, selectedRows, rightSiderFold, rightSiderNodeId, dispatch, loading, rscOperateLoading, tableHeight } = this.props

    let columns = [{
      title: '资源名',
      dataIndex: 'id',
      key: 'id',
      sorter: (a, b) => {
        return strcmp(a.id, b.id)
      },
    }, {
      title: '状态',
      dataIndex: 'status',
      key: 'status',
      sorter: (a, b) => {
        return strcmp(a.status, b.status)
      },
      width: '10%',
      render: (status, record) => {
        let tooltipTitle = record.status_message && status === 'Not Running' ? 'Failed' : status

        if (record.drbdStatus && record.drbdStatus !== '0') {
          tooltipTitle = record.drbdMsg
        }

        return (
          <Tooltip placement="right" title={tooltipTitle}>
            <span style={{ marginLeft: '10%' }}>
              { rscOperateLoading && selectedRows[0].id === record.id ?
                <Icon type="loading" className={styles.statusLoading} /> : <Badge className={record.drbdStatus && record.drbdStatus !== '0' ? styles.redTwinkling : ''} status={parseRscStatusMsg(record)} />
              }
            </span>
          </Tooltip>
        )
      },
    }, {
      title: '资源类型',
      dataIndex: 'type',
      key: 'type',
      width: '15%',
      sorter: (a, b) => {
        return strcmp(a.type, b.type)
      },
    }, {
      title: '服务',
      dataIndex: 'svc',
      key: 'svc',
      width: '15%',
      sorter: (a, b) => {
        return strcmp(a.svc || '', b.svc || '')
      },
    }]

    nodelist.forEach((node) => {
      parseNodeStatusMsg(node)
      columns.push({
        title: <a onClick={() => dispatch({ type: 'resource/updateState', payload: { rightSiderFold: false, rightSiderNodeId: node.id } })}>
          <Tooltip placement="top" title={node.status}>
            <Badge dot={node.standby} style={{ backgroundColor: 'orange' }}>
              <Avatar size={'small'} icon={'desktop'} style={{ color: 'white', backgroundColor: node.running ? 'green' : 'red' }} />
            </Badge>
          </Tooltip>
          <span className={node.id === rightSiderNodeId && !rightSiderFold ? styles.selectedNode : ''}>&nbsp;{ node.id }</span>
        </a>,
        dataIndex: 'running_node',
        key: node.id,
        width: `${36 / nodelist.length}%`,
        render: (runningNode, record) => {
          let location = queryArray(record.location, node.id, 'node') || {}
          let status = (runningNode.indexOf(node.id) === -1) ? 'default' : 'success'
          return (
            <span style={{ marginLeft: '10%' }}>
              <Badge status={status} text={location.level} />
            </span>
          )
        },
      })
    })

    const rowSelection = {
      selectedRowKeys,
      onChange: (keys, rows) => {
        dispatch({
          type: 'resource/updateState',
          payload: {
            selectedRowKeys: keys,
            selectedRows: rows,
          },
        })
      },
      type: 'radio',
    }

    const rscMenu = (
      <Menu onClick={(e) => {
        dispatch({ type: 'resource/updateState', payload: { detailModalVisible: true, metaAttrs: {}, rsc: { category: e.key } } })
        if (e.key === 'primitive') {
          dispatch({ type: 'resource/queryRscTypes' })
        }
        dispatch({ type: 'resource/queryRscAttrs', payload: { category: e.key } })
      }}
      >
        <Menu.Item key="primitive">添加普通资源</Menu.Item>
        <Menu.Item key="group">添加组资源</Menu.Item>
        <Menu.Item key="clone">添加克隆资源</Menu.Item>
      </Menu>
    )

    const relationMenu = (
      <Menu onClick={(e) => {
        if (e.key === 'location') {
          dispatch({ type: 'node/query' })
        } else {
          dispatch({ type: 'resource/query' })
        }
        dispatch({ type: 'resource/queryRscRestriction', payload: { id: selectedRows[0].id, relation: e.key } }).then(() => {
          dispatch({ type: 'resource/updateState', payload: { rscRestrictionModalVisible: true, rscRestrictionModalTitle: e.key } })
        })
      }}
      >
        <Menu.Item key="location">资源位置</Menu.Item>
        <Menu.Item key="colocation">资源协同</Menu.Item>
        <Menu.Item key="order">资源顺序</Menu.Item>
      </Menu>
    )

    const operations = [
      {
        onClick: () => {
          if (selectedRows[0].type === 'primitive') {
            dispatch({ type: 'resource/queryRscTypes' })
          }
          dispatch({ type: 'resource/queryRscAttrs', payload: { category: selectedRows[0].type } })
          dispatch({ type: 'resource/detail', payload: { id: selectedRows[0].id.split(':')[0], category: selectedRows[0].type } }).then(() => {
            dispatch({ type: 'resource/updateState', payload: { detailModalVisible: true } })
          })
        },
        icon: 'edit',
        text: '编辑',
        enabled: () => { // DRBD主从资源及其子资源不支持编辑操作
          return selectedRowKeys.length === 1 && selectedRows[0].id !== 'ms-drbd' && selectedRows[0].id !== 'drbd-ps:0' && selectedRows[0].id !== 'drbd-ps:1'
        },
      }, {
        onClick: () => this.handleResourceOperation(selectedRows[0].id, 'start'),
        icon: 'caret-right',
        text: '启动',
        enabled: () => {
          let rows = selectedRows
          return rows.length === 1 && rows[0].status !== 'Running'
        },
      }, {
        onClick: () => this.handleResourceOperation(selectedRows[0].id, 'stop'),
        icon: 'stop',
        text: '停止',
        enabled: () => {
          let rows = selectedRows
          return rows.length === 1
        },
      }, {
        onClick: () => this.handleResourceOperation(selectedRows[0].id, 'cleanup'),
        icon: 'clear',
        text: '清理',
        enabled: () => {
          return selectedRowKeys.length === 1
        },
      }, {
        onClick: () => {
          dispatch({ type: 'resource/updateState', payload: { rscMigrationModalVisible: true } })
        },
        icon: 'export',
        text: '迁移',
        enabled: () => {
          let rows = selectedRows
          return rows.length === 1 && rows[0].status === 'Running' && rows[0].type !== 'clone' && !rows[0].isChild
        },
      }, {
        onClick: () => this.handleResourceOperation(selectedRows[0].id, 'unmigrate', { is_all_rscs: false }),
        icon: 'import',
        text: '回迁',
        enabled: () => {
          let rows = selectedRows
          return rows.length === 1 && rows[0].status === 'Running' && rows[0].type !== 'clone' && !rows[0].isChild && rows[0].allow_unmigrate
        },
      },
    ]

    let widthRange = document.body.clientWidth > 1280
    return (
      <div className={classnames(styles.container, { [styles.diminish]: !rightSiderFold && document.body.clientWidth > 425 })}>
        {/* loading &&  */}
        <Loader spinning={loading} />
        <div style={{ marginBottom: 0 }}>
          <Button.Group>
            <Dropdown overlay={rscMenu}>
              <Button><Icon type="plus" />添加...</Button>
            </Dropdown>
            {
              operations.map(op =>
                (<Button key={op.text} disabled={!op.enabled()} onClick={op.onClick} title={op.text}>
                  <Icon type={op.icon} />
                  { widthRange && op.text}
                </Button>)
              )
            }
            <Popconfirm
              placement="bottom"
              onConfirm={() => {
                this.handleResourceOperation(selectedRows[0].id, 'delete')
              }}
              title="确认删除资源?"
            >
              <Button disabled={!(selectedRowKeys.length === 1 && !selectedRows[0].isChild)} title={'删除'}>
                <Icon type="delete" />
                { widthRange && '删除'}
              </Button>
            </Popconfirm>
            <Dropdown disabled={!(selectedRowKeys.length === 1 && selectedRows[0].type !== 'clone' && !selectedRows[0].isChild && selectedRows[0].id !== 'ms-drbd')} overlay={relationMenu}>
              <Button><Icon type="sharealt" />关系..</Button>
            </Dropdown>
          </Button.Group>
          <Button title={'刷新'} style={{ float: document.body.clientWidth > 425 ? 'right' : undefined }} onClick={this.handleReFresh.bind(this)}>
            <Icon type="reload" />
            { widthRange && '刷新'}
          </Button>

        </div>
        <Table
          columns={columns}
          rowSelection={rowSelection}
          dataSource={rsclist}
          scroll={{ y: tableHeight }}
          pagination={false}
          onRow={(record) => {
            return {
              onClick: () => this.handleRowClick(record),
            }
          }}
          locale={{ emptyText: '暂无数据' }}
        />
        <div className={classnames(styles.rightSider, { [styles.rightSiderFold]: rightSiderFold })}>
          <div className={styles.button} onClick={() => dispatch({ type: 'resource/updateState', payload: { rightSiderFold: true } })} >
            <Icon type="right" />
          </div>
          <NodeView />
        </div>
        <RscDetailModal />
        <RscMigrationModal />
        <RscRestrictions />
      </div>
    )
  }
}

Resource.propTypes = {
  dispatch: PropTypes.func,
  rsclist: PropTypes.array,
  nodelist: PropTypes.array,
  selectedRowKeys: PropTypes.array,
  selectedRows: PropTypes.array,
  rightSiderFold: PropTypes.bool,
  loading: PropTypes.bool,
  rscOperateLoading: PropTypes.bool,
  tableHeight: PropTypes.number,
  mainLockVisible: PropTypes.bool,
}

const mapStateToProps = ({ resource, node, loading, app }) => {
  return {
    rsclist: resource.list,
    nodelist: node.list,
    selectedRowKeys: resource.selectedRowKeys,
    selectedRows: resource.selectedRows,
    rightSiderFold: resource.rightSiderFold,
    rightSiderNodeId: resource.rightSiderNodeId,
    loading: loading.effects['resource/firstQuery'],
    rscOperateLoading: loading.effects['resource/operate'],
    tableHeight: document.body.clientHeight - 200,
    mainLockVisible: resource.mainLockVisible,
    autoReFreshId: app.autoReFreshId,
    autoReFreshTaskId: app.autoReFreshTaskId,
  }
}

export default connect(mapStateToProps)(Resource)
