/* global window, document */
import { message } from 'antd'
import config from './config'
import request from './request'
import { color } from './theme'

let strcmp = function (str1, str2) {
  if (str1 === str2) {
    return 0
  }

  return (str1 > str2) ? 1 : -1
}

/**
 * @param   {String}
 * @return  {String}
 */

const queryURL = (name) => {
  let reg = new RegExp(`(^|&)${name}=([^&]*)(&|$)`, 'i')
  let r = window.location.search.substr(1).match(reg)
  if (r != null) return decodeURI(r[2])
  return null
}

/**
 * 数组内查询
 * @param   {array}      array
 * @param   {String}    id
 * @param   {String}    keyAlias
 * @return  {Array}
 */
const queryArray = (array, key, keyAlias = 'key') => {
  if (!(array instanceof Array)) {
    return null
  }
  const item = array.filter(_ => _[keyAlias] === key)
  if (item.length) {
    return item[0]
  }
  return null
}

const transformInfinity = (value) => {
  switch (value) {
    case '-INFINITY':
      return -1000000
    case 'INFINITY':
      return 1000000
    default:
      return value
  }
}

const transformBoolean = (value) => {
  if (typeof value !== 'string') {
    return Boolean(value)
  }
  switch (value.toLowerCase()) {
    case 'true':
    case 'on':
    case 'yes':
    case 'y':
    case '1':
      return true
    case 'false':
    case 'off':
    case 'no':
    case 'n':
    case '0':
      return false
    default:
      return value
  }
}

const transformData = (params) => {
  Object.keys(params).forEach((key) => {
    let param = params[key]
    if (param.content.type === 'integer') {
      param.value = transformInfinity(param.value)
      param.content.default = transformInfinity(param.content.default)
    } else if (param.content.type === 'boolean') {
      param.value = transformBoolean(param.value)
      param.content.default = transformBoolean(param.content.default)
    }
  })
}

const getLicenseType = (licenseModel) => {
  if (!licenseModel.is_active) {
    licenseModel.type = '未激活'
    return licenseModel
  }

  if (licenseModel.version_id > 0) {
    licenseModel.type = '正式版'
  } else if (licenseModel.version_id === 0 && licenseModel.halicense_is_expired === 0) {
    let leftTime = Math.ceil((new Date(licenseModel.valid_period.replace(' ', 'T')).getTime() - new Date(licenseModel.now_time.replace(' ', 'T')).getTime()) / (1000 * 3600 * 24))
    licenseModel.type = `试用版(剩余${leftTime}天)`
  } else {
    licenseModel.type = '版本过期'
  }

  return licenseModel
}

const validateDescriptor = {
  required: { required: true, message: '请输入该必填项' },
  email: { type: 'email', message: '请输入有效的邮箱地址' },
  smtp: { pattern: /^([a-zA-Z0-9])+(\.[a-zA-Z0-9])+/, message: '请输入合法的SMTP服务器名称' },
  ip: { pattern: /^([1-9]|[1-9]\d|1\d{2}|2[0-1]\d|22[0-3])(\.(\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])){3}$/, message: '请输入有效的IP地址' },
  multiCastIp: { pattern: /^(22[5-9]|23[0-9])(\.(\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])){3}$/, message: '请输入D类多播地址，范围为225.0.0.0~239.255.255.255' },
  netSegment: { pattern: /^([1-9]|[1-9]\d|1\d{2}|2[0-1]\d|22[0-3])(\.(\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])){2}\.0$/, message: '请输入合法IP地址，且必须以0结尾' },
  alphanumeric: { pattern: /^[a-zA-Z0-9_-]*$/, message: '请勿输入除英文字母、数字、-、_之外的字符' },
  startWithAlpha: { pattern: /^[a-zA-Z]/, message: '请以字母开头' },
  absPath: { pattern: /^\//, message: '请以/开头' },
  devPath: { pattern: /^\/dev\//, message: '请以/dev/开头' },
  validPath: { pattern: /^[/.a-zA-Z0-9_-]*$/, message: '请勿输入除英文字母、数字、-、_、./之外的字符' },
  length: (min, max) => ({ min, max, message: `字符长度为${min}~${max}` }),
}

const handleOperation = (dispatch, namespace, id, action, data) => {
  return dispatch({ type: `${namespace}/operate`, payload: { id, action, ...data } }).then((resp) => {
    message.success(resp.info, 5)
    dispatch({ type: `${namespace}/query` })
    return Promise.resolve()
  })
}

const triggerEvent = (id, eventName) => {
  let select = document.getElementById(id)
  if (document.createEvent) {
    let ev = document.createEvent('HTMLEvents')
    ev.initEvent(eventName, true, false)
    select.dispatchEvent(ev)
  } else if (document.createEventObject) {
    select.fireEvent(eventName)
  }
}

const triggerTimedTask = (dispatch, timer) => {
  let intervalId = setInterval(() => {
    dispatch({ type: 'resource/query' })
    dispatch({ type: 'node/query' })
    dispatch({ type: 'cluster/queryHeartBeatStatus' })
  }, timer * 5000)

  return intervalId
}

const getDrbdEnable = (resData) => {
  let drbdRes = ['group-fs-ps', 'clone-gfs', 'ms-drbd']
  return resData.some((res) => {
    return drbdRes.indexOf(res.id) > -1
  })
}

module.exports = {
  config,
  request,
  color,
  queryURL,
  queryArray,
  strcmp,
  transformData,
  validateDescriptor,
  handleOperation,
  getLicenseType,
  triggerEvent,
  triggerTimedTask,
  transformBoolean,
  getDrbdEnable,
}
